# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import TYPE_CHECKING, Any, Mapping
from typing_extensions import Self, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
    not_given,
)
from ._utils import is_given, get_async_library
from ._compat import cached_property
from ._version import __version__
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import Channel3Error, APIStatusError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)

if TYPE_CHECKING:
    from .resources import brands, enrich, search, products, websites, price_tracking
    from .resources.brands import BrandsResource, AsyncBrandsResource
    from .resources.enrich import EnrichResource, AsyncEnrichResource
    from .resources.search import SearchResource, AsyncSearchResource
    from .resources.products import ProductsResource, AsyncProductsResource
    from .resources.websites import WebsitesResource, AsyncWebsitesResource
    from .resources.price_tracking import PriceTrackingResource, AsyncPriceTrackingResource

__all__ = [
    "Timeout",
    "Transport",
    "ProxiesTypes",
    "RequestOptions",
    "Channel3",
    "AsyncChannel3",
    "Client",
    "AsyncClient",
]


class Channel3(SyncAPIClient):
    # client options
    api_key: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous Channel3 client instance.

        This automatically infers the `api_key` argument from the `CHANNEL3_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("CHANNEL3_API_KEY")
        if api_key is None:
            raise Channel3Error(
                "The api_key client option must be set either by passing api_key to the client or by setting the CHANNEL3_API_KEY environment variable"
            )
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("CHANNEL3_BASE_URL")
        if base_url is None:
            base_url = f"https://api.trychannel3.com"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def search(self) -> SearchResource:
        from .resources.search import SearchResource

        return SearchResource(self)

    @cached_property
    def products(self) -> ProductsResource:
        from .resources.products import ProductsResource

        return ProductsResource(self)

    @cached_property
    def brands(self) -> BrandsResource:
        from .resources.brands import BrandsResource

        return BrandsResource(self)

    @cached_property
    def websites(self) -> WebsitesResource:
        from .resources.websites import WebsitesResource

        return WebsitesResource(self)

    @cached_property
    def enrich(self) -> EnrichResource:
        from .resources.enrich import EnrichResource

        return EnrichResource(self)

    @cached_property
    def price_tracking(self) -> PriceTrackingResource:
        from .resources.price_tracking import PriceTrackingResource

        return PriceTrackingResource(self)

    @cached_property
    def with_raw_response(self) -> Channel3WithRawResponse:
        return Channel3WithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> Channel3WithStreamedResponse:
        return Channel3WithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"x-api-key": api_key}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncChannel3(AsyncAPIClient):
    # client options
    api_key: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncChannel3 client instance.

        This automatically infers the `api_key` argument from the `CHANNEL3_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("CHANNEL3_API_KEY")
        if api_key is None:
            raise Channel3Error(
                "The api_key client option must be set either by passing api_key to the client or by setting the CHANNEL3_API_KEY environment variable"
            )
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("CHANNEL3_BASE_URL")
        if base_url is None:
            base_url = f"https://api.trychannel3.com"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def search(self) -> AsyncSearchResource:
        from .resources.search import AsyncSearchResource

        return AsyncSearchResource(self)

    @cached_property
    def products(self) -> AsyncProductsResource:
        from .resources.products import AsyncProductsResource

        return AsyncProductsResource(self)

    @cached_property
    def brands(self) -> AsyncBrandsResource:
        from .resources.brands import AsyncBrandsResource

        return AsyncBrandsResource(self)

    @cached_property
    def websites(self) -> AsyncWebsitesResource:
        from .resources.websites import AsyncWebsitesResource

        return AsyncWebsitesResource(self)

    @cached_property
    def enrich(self) -> AsyncEnrichResource:
        from .resources.enrich import AsyncEnrichResource

        return AsyncEnrichResource(self)

    @cached_property
    def price_tracking(self) -> AsyncPriceTrackingResource:
        from .resources.price_tracking import AsyncPriceTrackingResource

        return AsyncPriceTrackingResource(self)

    @cached_property
    def with_raw_response(self) -> AsyncChannel3WithRawResponse:
        return AsyncChannel3WithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncChannel3WithStreamedResponse:
        return AsyncChannel3WithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"x-api-key": api_key}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class Channel3WithRawResponse:
    _client: Channel3

    def __init__(self, client: Channel3) -> None:
        self._client = client

    @cached_property
    def search(self) -> search.SearchResourceWithRawResponse:
        from .resources.search import SearchResourceWithRawResponse

        return SearchResourceWithRawResponse(self._client.search)

    @cached_property
    def products(self) -> products.ProductsResourceWithRawResponse:
        from .resources.products import ProductsResourceWithRawResponse

        return ProductsResourceWithRawResponse(self._client.products)

    @cached_property
    def brands(self) -> brands.BrandsResourceWithRawResponse:
        from .resources.brands import BrandsResourceWithRawResponse

        return BrandsResourceWithRawResponse(self._client.brands)

    @cached_property
    def websites(self) -> websites.WebsitesResourceWithRawResponse:
        from .resources.websites import WebsitesResourceWithRawResponse

        return WebsitesResourceWithRawResponse(self._client.websites)

    @cached_property
    def enrich(self) -> enrich.EnrichResourceWithRawResponse:
        from .resources.enrich import EnrichResourceWithRawResponse

        return EnrichResourceWithRawResponse(self._client.enrich)

    @cached_property
    def price_tracking(self) -> price_tracking.PriceTrackingResourceWithRawResponse:
        from .resources.price_tracking import PriceTrackingResourceWithRawResponse

        return PriceTrackingResourceWithRawResponse(self._client.price_tracking)


class AsyncChannel3WithRawResponse:
    _client: AsyncChannel3

    def __init__(self, client: AsyncChannel3) -> None:
        self._client = client

    @cached_property
    def search(self) -> search.AsyncSearchResourceWithRawResponse:
        from .resources.search import AsyncSearchResourceWithRawResponse

        return AsyncSearchResourceWithRawResponse(self._client.search)

    @cached_property
    def products(self) -> products.AsyncProductsResourceWithRawResponse:
        from .resources.products import AsyncProductsResourceWithRawResponse

        return AsyncProductsResourceWithRawResponse(self._client.products)

    @cached_property
    def brands(self) -> brands.AsyncBrandsResourceWithRawResponse:
        from .resources.brands import AsyncBrandsResourceWithRawResponse

        return AsyncBrandsResourceWithRawResponse(self._client.brands)

    @cached_property
    def websites(self) -> websites.AsyncWebsitesResourceWithRawResponse:
        from .resources.websites import AsyncWebsitesResourceWithRawResponse

        return AsyncWebsitesResourceWithRawResponse(self._client.websites)

    @cached_property
    def enrich(self) -> enrich.AsyncEnrichResourceWithRawResponse:
        from .resources.enrich import AsyncEnrichResourceWithRawResponse

        return AsyncEnrichResourceWithRawResponse(self._client.enrich)

    @cached_property
    def price_tracking(self) -> price_tracking.AsyncPriceTrackingResourceWithRawResponse:
        from .resources.price_tracking import AsyncPriceTrackingResourceWithRawResponse

        return AsyncPriceTrackingResourceWithRawResponse(self._client.price_tracking)


class Channel3WithStreamedResponse:
    _client: Channel3

    def __init__(self, client: Channel3) -> None:
        self._client = client

    @cached_property
    def search(self) -> search.SearchResourceWithStreamingResponse:
        from .resources.search import SearchResourceWithStreamingResponse

        return SearchResourceWithStreamingResponse(self._client.search)

    @cached_property
    def products(self) -> products.ProductsResourceWithStreamingResponse:
        from .resources.products import ProductsResourceWithStreamingResponse

        return ProductsResourceWithStreamingResponse(self._client.products)

    @cached_property
    def brands(self) -> brands.BrandsResourceWithStreamingResponse:
        from .resources.brands import BrandsResourceWithStreamingResponse

        return BrandsResourceWithStreamingResponse(self._client.brands)

    @cached_property
    def websites(self) -> websites.WebsitesResourceWithStreamingResponse:
        from .resources.websites import WebsitesResourceWithStreamingResponse

        return WebsitesResourceWithStreamingResponse(self._client.websites)

    @cached_property
    def enrich(self) -> enrich.EnrichResourceWithStreamingResponse:
        from .resources.enrich import EnrichResourceWithStreamingResponse

        return EnrichResourceWithStreamingResponse(self._client.enrich)

    @cached_property
    def price_tracking(self) -> price_tracking.PriceTrackingResourceWithStreamingResponse:
        from .resources.price_tracking import PriceTrackingResourceWithStreamingResponse

        return PriceTrackingResourceWithStreamingResponse(self._client.price_tracking)


class AsyncChannel3WithStreamedResponse:
    _client: AsyncChannel3

    def __init__(self, client: AsyncChannel3) -> None:
        self._client = client

    @cached_property
    def search(self) -> search.AsyncSearchResourceWithStreamingResponse:
        from .resources.search import AsyncSearchResourceWithStreamingResponse

        return AsyncSearchResourceWithStreamingResponse(self._client.search)

    @cached_property
    def products(self) -> products.AsyncProductsResourceWithStreamingResponse:
        from .resources.products import AsyncProductsResourceWithStreamingResponse

        return AsyncProductsResourceWithStreamingResponse(self._client.products)

    @cached_property
    def brands(self) -> brands.AsyncBrandsResourceWithStreamingResponse:
        from .resources.brands import AsyncBrandsResourceWithStreamingResponse

        return AsyncBrandsResourceWithStreamingResponse(self._client.brands)

    @cached_property
    def websites(self) -> websites.AsyncWebsitesResourceWithStreamingResponse:
        from .resources.websites import AsyncWebsitesResourceWithStreamingResponse

        return AsyncWebsitesResourceWithStreamingResponse(self._client.websites)

    @cached_property
    def enrich(self) -> enrich.AsyncEnrichResourceWithStreamingResponse:
        from .resources.enrich import AsyncEnrichResourceWithStreamingResponse

        return AsyncEnrichResourceWithStreamingResponse(self._client.enrich)

    @cached_property
    def price_tracking(self) -> price_tracking.AsyncPriceTrackingResourceWithStreamingResponse:
        from .resources.price_tracking import AsyncPriceTrackingResourceWithStreamingResponse

        return AsyncPriceTrackingResourceWithStreamingResponse(self._client.price_tracking)


Client = Channel3

AsyncClient = AsyncChannel3
