# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["PriceTrackingGetHistoryParams"]


class PriceTrackingGetHistoryParams(TypedDict, total=False):
    days: int
    """Number of days of history to fetch (max 30)"""
